from PyQt6.QtWidgets import (
    QApplication, QMainWindow, QWidget, QVBoxLayout, 
    QPushButton, QLineEdit, QLabel, QMessageBox,
    QFileDialog, QHBoxLayout,
    QStatusBar, QStyle, QProgressBar
)
from PyQt6.QtCore import Qt, QTimer
from PyQt6.QtGui import QPalette, QColor, QIcon
import sys
import os
from pathlib import Path
from .installer import ProjectInstaller

class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("컨시언스파트너스 백엔드 시스템")
        self.setFixedSize(500, 300)
        self.setup_ui()
        self.setup_dark_theme()
        
    def setup_ui(self):
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        layout = QVBoxLayout(central_widget)
        layout.setSpacing(15)
        layout.setContentsMargins(20, 20, 20, 20)

        # 제목 추가
        title = QLabel("컨시언스파트너스 백엔드 시스템 - 공개 버전")
        title.setStyleSheet("font-size: 20px; font-weight: bold; margin-bottom: 24px;")
        layout.addWidget(title, alignment=Qt.AlignmentFlag.AlignCenter)

        # 프로젝트 경로 선택
        path_layout = QHBoxLayout()
        self.path_input = QLineEdit()
        self.path_input.setText(str(Path.cwd()))
        self.path_input.setPlaceholderText("프로젝트 경로 선택...")
        self.path_input.setToolTip("프로젝트가 생성될 경로를 선택하세요")
        
        # 버튼 스타일 정의
        button_style = """
            QPushButton {
                background-color: #2980b9;
                color: white;
                border: none;
                padding: 8px 15px;
                border-radius: 4px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #3498db;
            }
            QPushButton:pressed {
                background-color: #2475a8;
            }
            QPushButton:disabled {
                background-color: #bdc3c7;
                color: #7f8c8d;
            }
        """

        # 경로 선택 버튼
        self.path_button = QPushButton("경로 선택")
        self.path_button.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_DirIcon))
        self.path_button.clicked.connect(self.select_path)
        self.path_button.setStyleSheet(button_style)
        
        path_layout.addWidget(self.path_input)
        path_layout.addWidget(self.path_button)

        # 서비스 이름 입력
        self.name_input = QLineEdit()
        self.name_input.setPlaceholderText("서비스 이름 (예: my_service)")
        self.name_input.setToolTip("영문, 숫자, 언더스코어만 사용 가능합니다")
        self.name_input.textChanged.connect(self.validate_inputs)

        # Database URL 입력
        self.url_input = QLineEdit()
        self.url_input.setPlaceholderText("postgresql://user:password@host:port/dbname")
        self.url_input.setToolTip("PostgreSQL 연결 문자열을 입력하세요")
        self.url_input.textChanged.connect(self.validate_inputs)

        # 실행 버튼
        self.setup_button = QPushButton("서비스 설정 및 실행")
        self.setup_button.setEnabled(False)
        self.setup_button.setFixedHeight(45)  # 버튼 높이 고정
        self.setup_button.setStyleSheet("""
            QPushButton {
                background-color: #2980b9;
                color: white;
                border: none;
                padding: 12px;
                border-radius: 4px;
                font-weight: bold;
                font-size: 14px;
                text-align: center;
                position: relative;
            }
            QPushButton:hover {
                background-color: #3498db;
            }
            QPushButton:pressed {
                background-color: #2475a8;
            }
            QPushButton:disabled {
                background-color: #2980b9;
                color: white;
            }
        """)
        self.setup_button.clicked.connect(self.setup_system)

        # 프로그레스바 생성
        self.progress_bar = QProgressBar(self.setup_button)
        self.progress_bar.setTextVisible(False)
        self.progress_bar.setStyleSheet("""
            QProgressBar {
                border: none;
                background-color: transparent;
                border-radius: 4px;
            }
            QProgressBar::chunk {
                background-color: rgba(52, 152, 219, 0.5);
                border-radius: 4px;
            }
        """)
        self.progress_bar.hide()  # 초기에는 숨김

        # 레이아웃에 위젯 추가
        layout.addLayout(path_layout)
        layout.addWidget(self.name_input)
        layout.addWidget(self.url_input)
        layout.addWidget(self.setup_button)
        layout.addStretch()

        # 상태 표시줄 추가
        self.status_bar = QStatusBar()
        self.setStatusBar(self.status_bar)
        self.status_bar.showMessage("시스템 준비 완료")

    def setup_dark_theme(self):
        palette = QPalette()
        palette.setColor(QPalette.ColorRole.Window, QColor(53, 53, 53))
        palette.setColor(QPalette.ColorRole.WindowText, Qt.GlobalColor.white)
        palette.setColor(QPalette.ColorRole.Base, QColor(25, 25, 25))
        palette.setColor(QPalette.ColorRole.AlternateBase, QColor(53, 53, 53))
        palette.setColor(QPalette.ColorRole.ToolTipBase, Qt.GlobalColor.white)
        palette.setColor(QPalette.ColorRole.ToolTipText, Qt.GlobalColor.white)
        palette.setColor(QPalette.ColorRole.Text, Qt.GlobalColor.white)
        palette.setColor(QPalette.ColorRole.Button, QColor(53, 53, 53))
        palette.setColor(QPalette.ColorRole.ButtonText, Qt.GlobalColor.white)
        palette.setColor(QPalette.ColorRole.Link, QColor(42, 130, 218))
        palette.setColor(QPalette.ColorRole.Highlight, QColor(42, 130, 218))
        palette.setColor(QPalette.ColorRole.HighlightedText, Qt.GlobalColor.black)
        
        self.setPalette(palette)

    def validate_inputs(self):
        name = self.name_input.text()
        url = self.url_input.text()
        
        name_valid = bool(name and name.replace('_', '').isalnum())
        url_valid = bool(url)  # URL이 비어있지만 않으면 됨
        
        self.setup_button.setEnabled(name_valid and url_valid)
        
        if name and not name_valid:
            self.status_bar.showMessage("서비스 이름은 영문, 숫자, 언더스코어만 사용 가능합니다")
        else:
            self.status_bar.showMessage("준비 완료")

    def select_path(self):
        folder = QFileDialog.getExistingDirectory(
            self,
            "프로젝트를 생성할 경로 선택",
            str(Path.home()),  # 기본 경로를 홈 디렉토리로 설정
            QFileDialog.Option.ShowDirsOnly
        )
        if folder:
            self.path_input.setText(folder)

    def setup_system(self):
        service_name = self.name_input.text()
        db_url = self.url_input.text()
        project_path = Path(self.path_input.text())
        
        if not service_name or not db_url:
            QMessageBox.warning(self, "경고", "모든 필드를 입력해주세요.")
            return

        # 버튼 비활성화 및 프로그레스바 초기화
        self.setup_button.setEnabled(False)
        original_text = self.setup_button.text()
        self.setup_button.setText("설치 중...")
        
        # 프로그레스바 설정
        self.progress_bar.setGeometry(0, 0, self.setup_button.width(), self.setup_button.height())
        self.progress_bar.setValue(0)
        self.progress_bar.show()

        try:
            os.chdir(project_path)
            installer = ProjectInstaller(service_name, db_url)
            
            # 각 단계별 실행 및 진행률 업데이트
            steps = [
                ("디렉토리 구조 생성 중...", installer.create_directory_structure),
                ("패키지 설치 중...", installer.install_requirements),
                ("가상환경 설정 중...", installer.setup_virtual_environment),
                ("마이그레이션 디렉토리 생성 중...", installer.create_migrations_directories),
                ("프로젝트 파일 생성 중...", installer.create_project_files),
                ("데이터베이스 설정 중...", installer.setup_database),
                ("Git 저장소 초기화 중...", installer.initialize_git)
            ]
            
            total_steps = len(steps)
            for i, (message, func) in enumerate(steps, 1):
                self.status_bar.showMessage(f"{i}/{total_steps} {message}")
                # 진행률을 정수로 변환
                progress = int((i / total_steps) * 100)
                self.progress_bar.setValue(progress)
                QApplication.processEvents()  # UI 업데이트
                
                func()  # 각 단계 실행
            
            # 새 터미널에서 서버 실행
            os.system(f'''osascript -e 'tell app "Terminal"
                do script "{installer.project_dir}/run.sh"
            end tell' ''')
            
            self.progress_bar.setValue(100)
            self.setup_button.setText("설치 완료")
            QApplication.processEvents()
            
            QMessageBox.information(
                self, 
                "성공", 
                "서비스가 성공적으로 설정되었습니다."
            )
        except Exception as e:
            QMessageBox.critical(self, "오류", f"설정 중 오류가 발생했습니다: {str(e)}")
        finally:
            # 잠시 후 버튼 상태 복구
            QTimer.singleShot(1500, lambda: self.reset_button_state(original_text))

    def reset_button_state(self, original_text):
        self.setup_button.setEnabled(True)
        self.setup_button.setText(original_text)
        self.progress_bar.hide()
        self.status_bar.showMessage("준비 완료")

    def resizeEvent(self, event):
        super().resizeEvent(event)
        if hasattr(self, 'progress_bar'):
            self.progress_bar.setGeometry(0, 0, self.setup_button.width(), self.setup_button.height())

def main():
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec()) 