import os
import shutil
import venv
import subprocess
from pathlib import Path
from django.core.management.utils import get_random_secret_key
import logging
import sys

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

class ProjectInstaller:
    def __init__(self, service_name: str, db_url: str):
        self.service_name = service_name
        self.db_url = db_url
        self.project_dir = Path.cwd() / service_name
        self.current_dir = Path(__file__).parent
        self.src_dir = self.current_dir / "src"
        self.venv_dir = self.project_dir / "venv"

    def create_directory_structure(self):
        """프로젝트 기본 디렉토리 구조 생성"""
        logger.info("Creating project directory structure...")
        self.project_dir.mkdir(exist_ok=True)
        
        if not self.src_dir.exists():
            raise FileNotFoundError(
                f"Source directory not found at {self.src_dir}. "
                "Please ensure the package is installed correctly."
            )
        
        shutil.copytree(self.src_dir, self.project_dir / "src", dirs_exist_ok=True)

    def install_requirements(self):
        """requirements.txt 복사"""
        logger.info("Copying requirements.txt...")
        requirements_path = self.project_dir / "src" / "requirements.txt"
        if requirements_path.exists():
            logger.info("Copying requirements.txt to project root...")
            shutil.copy2(requirements_path, self.project_dir / "requirements.txt")

    def setup_virtual_environment(self):
        """가상환경 설정"""
        logger.info("Creating virtual environment...")
        venv.create(self.venv_dir, with_pip=True)
        
        # 가상환경 활성화
        logger.info("Activating virtual environment...")
        activate_script = self.venv_dir / "bin" / "activate"
        activate_cmd = f"source {activate_script}"
        
        # 의존성 설치
        logger.info("Installing dependencies in virtual environment...")
        pip_path = self.venv_dir / "bin" / "pip"
        requirements_path = self.project_dir / "src" / "requirements.txt"
        
        # 활성화된 환경에서 pip 설치 실행
        subprocess.run(f"{activate_cmd} && {str(pip_path)} install -r {str(requirements_path)}", 
                     shell=True, executable='/bin/bash')
        subprocess.run(f"{activate_cmd} && {str(pip_path)} install rav", 
                     shell=True, executable='/bin/bash')

    def create_migrations_directories(self):
        """마이그레이션 디렉토리 설정"""
        logger.info("Creating migrations directories...")
        apps = ['user', 'commando']
        for app in apps:
            migrations_dir = self.project_dir / "src" / app / "migrations"
            migrations_dir.mkdir(exist_ok=True)
            (migrations_dir / "__init__.py").touch()

    def create_project_files(self):
        """프로젝트 설정 파일들 생성"""
        logger.info("Creating project configuration files...")
        self._create_dockerfile()
        self._create_rav_yaml()
        self._create_railway_toml()
        self._create_gitignore()
        self._create_env_file()
        self._create_readme()
        self._create_run_script()

    def setup_database(self):
        """데이터베이스 설정 및 마이그레이션"""
        logger.info("Setting up database...")
        python_path = self.venv_dir / "bin" / "python"
        manage_py = self.project_dir / "src" / "manage.py"

        # 기존 마이그레이션 파일 삭제
        migrations_cmd = f"find {self.project_dir}/src/*/migrations/ -type f -not -name '__init__.py' -delete"
        subprocess.run(migrations_cmd, shell=True)

        # 마이그레이션 생성 및 적용
        default_apps = ['admin', 'auth', 'contenttypes', 'sessions']
        custom_apps = ['user', 'commando']
        
        for app in default_apps + custom_apps:
            subprocess.run([str(python_path), str(manage_py), "makemigrations", app])
        
        subprocess.run([str(python_path), str(manage_py), "migrate", "--fake-initial"])

    def initialize_git(self):
        """Git 저장소 초기화"""
        logger.info("Initializing git repository...")
        subprocess.run(["git", "init"], cwd=self.project_dir)
        subprocess.run(["git", "add", "."], cwd=self.project_dir)
        subprocess.run(["git", "commit", "-m", "Initial commit"], cwd=self.project_dir)

    def _create_dockerfile(self):
        """Dockerfile 생성"""
        dockerfile_content = '''# Set the python version as a build-time argument
ARG PYTHON_VERSION=3.12-slim-bullseye
FROM python:${PYTHON_VERSION}

# Create a virtual environment
RUN python -m venv /opt/venv

# Set the virtual environment as the current location
ENV PATH=/opt/venv/bin:$PATH

# Upgrade pip
RUN pip install --upgrade pip

# Set Python-related environment variables
ENV PYTHONDONTWRITEBYTECODE 1
ENV PYTHONUNBUFFERED 1

# Install os dependencies
RUN apt-get update && apt-get install -y \\
    libpq-dev \\
    libjpeg-dev \\
    libcairo2 \\
    gcc \\
    && rm -rf /var/lib/apt/lists/*

# Create and set the working directory
RUN mkdir -p /code
WORKDIR /code

# Copy requirements and install dependencies
COPY requirements.txt /tmp/requirements.txt
COPY ./src /code

# Set environment variables
ARG DJANGO_SECRET_KEY
ENV DJANGO_SECRET_KEY=${DJANGO_SECRET_KEY}

ARG DJANGO_DEBUG=0
ENV DJANGO_DEBUG=${DJANGO_DEBUG}

# Install requirements
RUN pip install -r /tmp/requirements.txt
RUN pip install gunicorn

# Run initial commands
RUN python manage.py collectstatic --noinput

# Create runner script
RUN printf "#!/bin/bash\\n" > ./paracord_runner.sh && \\
    printf "RUN_PORT=\\"\\${PORT:-8000}\\"\\n\\n" >> ./paracord_runner.sh && \\
    printf "python manage.py migrate --no-input\\n" >> ./paracord_runner.sh && \\
    printf "gunicorn cfehome.wsgi:application --bind \\"0.0.0.0:\\$RUN_PORT\\"\\n" >> ./paracord_runner.sh

RUN chmod +x paracord_runner.sh

# Clean up
RUN apt-get remove --purge -y \\
    && apt-get autoremove -y \\
    && apt-get clean \\
    && rm -rf /var/lib/apt/lists/*

CMD ./paracord_runner.sh
'''
        (self.project_dir / "Dockerfile").write_text(dockerfile_content)

    def _create_rav_yaml(self):
        """rav.yaml 생성"""
        rav_content = '''scripts:
  server:
    - cd src && python manage.py runserver 8001
  makemigrations:
    - cd src && python manage.py makemigrations
  migrate:
    - cd src && python manage.py migrate
  startapp:
    - cd src && python manage.py startapp $1
  reset_db:
    - python reset_db.py
  shell:
    - cd src && python manage.py shell
  curl_auth: |
    curl -X POST -H "Content-Type: application/json" -d '{"username": "cfe", "password": "learncode"}' http://127.0.0.1:8001/api/token/pair
  curl_protect: |
    curl -X GET -H "Authorization: Bearer <curl_auth-access-token>" http://127.0.0.1:8001/api/me
  createsuperuser:
    - cd src && python manage.py createsuperuser
  test:
    - cd src && python manage.py test $1
  testfile:
    - cd src && python manage.py test $1.tests.$2
'''
        (self.project_dir / "rav.yaml").write_text(rav_content)

    def _create_railway_toml(self):
        """railway.toml 생성"""
        railway_content = '''[build]
builder = "DOCKERFILE"
dockerfilePath = "./Dockerfile"
watchPatterns = [
    "requirements.txt", 
    "src/**", 
    "railway.toml",
    "Dockerfile",
]
'''
        (self.project_dir / "railway.toml").write_text(railway_content)

    def _create_gitignore(self):
        """gitignore 파일 생성"""
        gitignore_content = """# Python
__pycache__/
*.py[cod]
*$py.class
*.so
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg

# Virtual Environment
venv/
ENV/

# Django
*.log
local_settings.py
db.sqlite3

# Environment variables
.env
.env.local
.env.*.local

# IDE
.idea/
.vscode/

# Docker
.docker/

# Node
node_modules/
"""
        (self.project_dir / ".gitignore").write_text(gitignore_content)

    def _create_env_file(self):
        """환경 변수 파일 생성"""
        secret_key = get_random_secret_key()
        env_content = f"""DJANGO_DEBUG=1
DJANGO_SECRET_KEY='{secret_key}'
DATABASE_URL='{self.db_url}'
DJANGO_ENV_NAME="local"
CORS_ALLOWED_ORIGINS="http://localhost:3000,http://127.0.0.1:3000"
ALLOWED_HOSTS="localhost,127.0.0.1"
DJANGO_SETTINGS_MODULE="cfehome.settings"
"""
        (self.project_dir / "src" / ".env").write_text(env_content)

    def _create_readme(self):
        """README.md 생성"""
        readme_content = f"""# {self.service_name}

## Setup
1. 가상환경 활성화:
```bash
source venv/bin/activate
```

2. 서버 실행:
```bash
cd src
rav run server
```

## Database Reset
PostgreSQL에서 다음 명령어 실행:
```sql
DROP SCHEMA public CASCADE;
CREATE SCHEMA public;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO public;
```

그 다음:
```bash
cd src
python manage.py migrate --fake-initial
```
"""
        (self.project_dir / "README.md").write_text(readme_content)

    def _create_run_script(self):
        """실행 스크립트 생성"""
        script_content = f"""#!/bin/bash
source {self.venv_dir}/bin/activate
cd {self.project_dir}/src
rav run server
"""
        run_script = self.project_dir / "run.sh"
        run_script.write_text(script_content)
        run_script.chmod(0o755)

def setup_project(service_name: str, db_url: str):
    """프로젝트 설정 메인 함수"""
    installer = ProjectInstaller(service_name, db_url)
    
    installer.create_directory_structure()
    installer.install_requirements()
    installer.setup_virtual_environment()
    installer.create_migrations_directories()
    installer.create_project_files()
    installer.setup_database()
    installer.initialize_git()

    # 새 터미널에서 스크립트 실행
    os.system(f'''osascript -e 'tell app "Terminal"
        do script "{installer.project_dir}/run.sh"
    end tell' ''')
    
    logger.info(f"""
Project setup completed successfully!
To run the server:
1. cd {service_name}
2. ./run.sh
""") 